/*
 * shell.h
 *
 * things having to do with shell fire
 *
 * Ken Stevens, 1996
 */

#ifndef _SHELL_H_
#define _SHELL_H_

#include "prototype.h"
#include "queue.h"
#include "genitem.h"
#include "land.h"

struct  targlist {
	struct qelem queue;
	struct shelist *target;
	int	distance;
};

struct  shelist {
        struct qelem queue;	/* linked list of items */
	struct genitem *item;	/* item to fire */
	int allocated;		/* was memory for item allocated? */
	int start_own;		/* the owner the item started with */
	int start_eff;		/* the efficiency item started at */
	int use_torp;		/* fire torpedoes */
	double damage_delivered;/* the amount of damage this item has done */
	double damage_received;	/* the amount of damage this item received */
	int distance;		/* distance to closest target */
	struct targlist targets;/* list of enemy items this item can hit */
	int ammo_cost;		/* number of shells consumed */
	double fire_range;	/* fire range */
	int rounds_left;	/* how many more times can I fire? */
	int (*canfire)();	/* can this item fire? */
	double (*range)();	/* fire range */
	int (*draw_supply)();	/* pull shells */
};

double tfactfire _PROTO((natid cn, double mult));
double techfactfire _PROTO((int level, double mult));
struct shelist *shell_create_item _PROTO((struct genitem *itemp, int ammo_cost, struct shelist *lp));
double shell_lnd_range _PROTO((struct lndstr *alandp));
double shell_shp_range _PROTO((struct shpstr *ashipp));
extern float fire_range_factor;

#define is_sub(itemp)  (itemp->ef_type == EF_SHIP && mchr[itemp->type].m_flags & M_SUB)

#define SHELL_MAX_GUNS 7		/* max number of guns that can shoot
					   at once */
#define SHELL_MAX_FIRE_RANGE 8		/* the max distance anything in the
					   game can fire */

#define SHELL_HOSTILE (-1)
#define SHELL_FIRE_COMMAND 1
#define SHELL_TORP_COMMAND 2
#define SHELL_RETURN_FIRE 3
#define SHELL_SHIP_INTERDICTION 4
#define SHELL_UNIT_INTERDICTION 5
#define SHELL_COMBAT_OSUPPORT 6
#define SHELL_COMBAT_DSUPPORT 7

#endif  /* _SHELL_H_ */
