#ifndef __EDBPRIV_H__
#define __EDBPRIV_H__

/*
	ObjectDescriptor has information necessary to construct and store
	the objects described by csync messages
*/

typedef	void ObjectStoreFunc _PROTO((void *, Empire_Database *, struct genobject *));
typedef	struct genobject *ObjectFindFunc _PROTO((void *, Empire_Database *, struct genobject *));
typedef	void ObjectPutVarFunc _PROTO((int, int, struct genobject *));
typedef	int ObjectGetVarFunc _PROTO((int, struct genobject *));

typedef struct {
	long objtype;
	char *objdescr;
	char *strname;		/* name of struct this describes */
	char *nscname;		/* nsc table describing this struct */
	char *fldprefix;	/* prefix for field names */
	int hasVarfields;
	int objsize;
	struct castr *obj_ca;
	ObjectStoreFunc *store;
	ObjectFindFunc *find;
	ObjectPutVarFunc *putvar;
	ObjectGetVarFunc *getvar;
	int collection_offset;	/* field offset in edb of collection */
	int index_offset;	/* field offset in edb of index */
	int key1, key2;		/* field offset in structure of keys */
	int index2_offset;
	int index2_key1, index2_key2;
} ObjectDescriptor;

/*
	FieldStruct is used to describe individual fields of objects
*/
typedef struct FieldStruct {
	struct FieldStruct *next;
	char *fldname;
	int  fldtype;
	int  fldcount;		/* number of elements if an array */
	struct castr *boundto;
} Field;

/*
	DictionaryStruct is used to describe each object in the database
*/
typedef struct DictionaryStruct {
	struct DictionaryStruct *next;
	int objnum;
	Field *flds;
	ObjectDescriptor *objdesc;
} Dictionary;

extern ObjectFindFunc find_object_i;
extern ObjectFindFunc find_object_int;
extern ObjectFindFunc find_object_s;
extern ObjectFindFunc find_object_uchar;
extern ObjectFindFunc find_object_nokey;
extern ObjectStoreFunc store_object_i;
extern ObjectStoreFunc store_object_int;
extern ObjectStoreFunc store_object_s;
extern ObjectStoreFunc store_object_uchar;
extern ObjectStoreFunc store_object_nokey;

#ifndef SERVERTYPES
#define sctstr_putvar 0
#define sctstr_getvar 0
#define lndcls_putvar 0
#define lndcls_getvar 0
#define lndstr_putvar 0
#define lndstr_getvar 0
#define shpstr_putvar 0
#define shpstr_getvar 0
#else
extern ObjectGetVarFunc 
	sctstr_getvar, lndcls_getvar, lndstr_getvar, shpstr_getvar;
extern ObjectPutVarFunc
	sctstr_putvar, lndcls_putvar, lndstr_putvar, shpstr_putvar;
#endif


#endif
