#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>

#include "edbproc.h"

/* edb_new -- allocate a new database control structure */
Empire_Database *edb_new()
{
	Empire_Database *p;
	p = (Empire_Database *) malloc(sizeof(Empire_Database));
	if (p) {
		memset(p, 0, sizeof(Empire_Database));
		p->objbuflen=EDB_OBJECT_BUFSZ;
		p->objbuffer = (char *) malloc(p->objbuflen);
		if (p->objbuffer)
			memset(p->objbuffer, 0, p->objbuflen);
	}
	return p;
}


/* edb_save -- save the database */
int edb_save(edb, func)
	Empire_Database  *edb;
	Edb_WriteFunc func;
{
	printf("edb_save called\n");
	return 0;
}


/* edb_load -- load the database */
int edb_load(edb, func)
	Empire_Database *edb;
	Edb_ReadFunc func;
{
	printf("edb_load called\n");
	return 0;
}

/* edb_getSectorBuild -- retrieve info about a particular sector type */
Sector_Build *edb_getSectorBuild(edb, n)
	Empire_Database *edb;
	int n;
{
	assert(edb!=NULL);
	return (Sector_Build *) getCollectionItem(edb->sector_build, n);
}

/* edb_getPlaneBuild -- retrieve info about a particular plane type */
Plane_Build *edb_getPlaneBuild(edb, planeType)
	Empire_Database *edb;
	int planeType;
{
	assert(edb!=NULL);
	return (Plane_Build *) getCollectionItem(edb->plane_build, planeType);
}

/* edb_getLandBuild -- retrieve info about a particular land type */
Land_Build *edb_getLandBuild(edb, landType)
	Empire_Database *edb;
	int landType;
{
	assert(edb!=NULL);
	return (Land_Build *) getCollectionItem(edb->land_build, landType);
}

/* edb_getShipBuild -- retrieve info about a particular ship type */
Ship_Build *edb_getShipBuild(edb, shipType)
	Empire_Database *edb;
	int shipType;
{
	assert(edb!=NULL);
	return (Ship_Build *) getCollectionItem(edb->ship_build, shipType);
}

/* edb_getNukeBuild -- retrieve info about a particular nuke type */
Nuke_Build *edb_getNukeBuild(edb, nukeType)
	Empire_Database *edb;
	int nukeType;
{
	assert(edb!=NULL);
	return (Nuke_Build *) getCollectionItem(edb->nuke_build, nukeType);
}

/* edb_getItemClass -- retrieve info about a particular commod type */
Item_Class *edb_getItemClass(edb, itemType)
	Empire_Database *edb;
	int itemType;
{
	assert(edb!=NULL);
	return (Item_Class *) getCollectionItem(edb->item_class, itemType);
}

/* edb_getItemClass -- retrieve info about a particular product type */
Product_Class *edb_getProductClass(edb, productType)
	Empire_Database *edb;
	int productType;
{
	assert(edb!=NULL);
	return (Product_Class *) getCollectionItem(edb->product_class, productType);
}

/* edb_getPlane -- retrieve info about a particular plane */
Plane  *edb_getPlane(edb, planeNo)
	Empire_Database *edb;
	int planeNo;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->plane_data_index, planeNo, 0);
	return (Plane *) getCollectionItem(edb->plane_data, slot);
}

/* edb_getLand -- retrieve info about a particular land unit */
Land   *edb_getLand(edb, unitNo)
	Empire_Database *edb;
	int unitNo;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->land_data_index, unitNo, 0);
	return (Land *) getCollectionItem(edb->land_data, slot);
}

/* edb_getShip -- retrieve info about a particular ship */
Ship   *edb_getShip(edb, shipNo)
	Empire_Database *edb;
	int shipNo;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->ship_data_index, shipNo, 0);
	return (Ship *) getCollectionItem(edb->ship_data, slot);
}

/* edb_getNuke -- retrieve info about a particular nuke */
Nuke   *edb_getNuke(edb, nukeNo)
	Empire_Database *edb;
	int nukeNo;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->ship_data_index, nukeNo, 0);
	return (Nuke *) getCollectionItem(edb->nuke_data, slot);
}

/* edb_getSector -- retrieve info about a particular sector */
Sector *edb_getSector(edb, x, y)
	Empire_Database *edb;
	int x;
	int y;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->sector_data_index, x, y);
	return (Sector *) getCollectionItem(edb->sector_data, slot);
}

/* edb_getCountry -- retrieve info about the country */
Country *edb_getCountry(edb)
	Empire_Database *edb;
{
	assert(edb!=NULL);
	return (Country *) getCollectionItem(edb->country_data, 0);
}

/* edb_getVersion -- retrieve info about the game */
Version *edb_getVersion(edb)
	Empire_Database *edb;
{
	assert(edb!=NULL);
	return (Version *) getCollectionItem(edb->game_data, 0);
}

/* edb_getMap -- retrieve map info */
Cmap *edb_getCmap(edb, x, y)
	Empire_Database *edb;
	int x;
	int y;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->sector_data_index, x, y);
	return (Cmap *) getCollectionItem(edb->map_data, slot);
}

/* edb_getBmap -- retrieve map info */
Bmap *edb_getBmap(edb, row)
	Empire_Database *edb;
	int row;
{
	int slot;
	assert(edb!=NULL);
	slot = findIndexEntry(edb->bmap_data_index, row, 0);
	return (Bmap *) getCollectionItem(edb->bmap_data, slot);
}

Collection *objectCollection(edb, itemType)
	Empire_Database *edb;
	EmpireObjectType itemType;
{
	assert(edb!=NULL);
	switch (itemType) {
		case V_Plane_Build:
			return edb->plane_build;
		case V_Land_Build:
			return edb->land_build;
		case V_Ship_Build:
			return edb->ship_build;
		case V_Sector_Build:
			return edb->sector_build;
		case V_Nuke_Build:
			return edb->nuke_build;
		case V_Item_Class:
			return edb->item_class;
		case V_Product_Class:
			return edb->product_class;
		case V_Plane_Data:
			return edb->plane_data;
		case V_Land_Data:
			return edb->land_data;
		case V_Ship_Data:
			return edb->ship_data;
		case V_Sector_Data:
			return edb->sector_data;
		case V_Country_Data:
			return edb->country_data;
		case V_Bmap_Data:
			return edb->bmap_data;
		case V_Cmap_Data:
			return edb->map_data;
		case V_Version_Data:
			return edb->game_data;
		case V_News_Data:
			return edb->news_data;
		case V_Trade_Data:
			return edb->trade_data;
		case V_Loan_Data:
			return edb->loan_data;
		case V_Treaty_Data:
			return edb->treaty_data;
		case V_Commodity_Data:
			return edb->commod_data;
	}
	return NULL;
}

void edb_enumerate(edb, itemType, func, context)
	Empire_Database *edb;
	EmpireObjectType itemType;
	Edb_CallBack func;
	void *context;
{
	int n, i;
	void *p;
	Collection *c;
	assert(edb!=NULL);
	c = objectCollection(edb, itemType);
	if (!c) return;
	n = getCollectionCount(c);
	for (i=0;i<n;i++) {
		p =  getCollectionItem(c,i);
		if (!p) continue;
		func(context,p);
	}
}


/* edb_getPlanesInSector -- retrieve info about planes in a sector */
Collection  *edb_getPlanesInSector(edb, x, y)
	Empire_Database *edb;
	int x;
	int y;
{
	/* this will be faster when we implement a secondary index */
	Collection *c, *r;
	int i,n;
	Plane *p;
	assert(edb!=NULL);
	r = newCollection(20,20);
	c = edb->plane_data;
	n = getCollectionCount(c);
	for (i=0;i<n;i++) {
		p = (Plane *) getCollectionItem(c, i);
		if (!p) continue;
		if (p->pln_x!=x || p->pln_y!=y) continue;
		addCollectionItem(r, p);
	}
	return r;
}

/* edb_getShipsInSector -- retrieve info about planes in a sector */
Collection  *edb_getShipsInSector(edb, x, y)
	Empire_Database *edb;
	int x;
	int y;
{
	/* this will be faster when we implement a secondary index */
	Collection *c, *r;
	int i,n;
	Ship *p;
	assert(edb!=NULL);
	r = newCollection(20,20);
	c = edb->ship_data;
	n = getCollectionCount(c);
	for (i=0;i<n;i++) {
		p = (Ship *) getCollectionItem(c, i);
		if (!p) continue;
		if (p->shp_x!=x || p->shp_y!=y) continue;
		addCollectionItem(r, p);
	}
	return r;
}

/* edb_getLandsInSector -- retrieve info about units in a sector */
Collection  *edb_getLandsInSector(edb, x, y)
	Empire_Database *edb;
	int x;
	int y;
{
	/* this will be faster when we implement a secondary index */
	Collection *c, *r;
	int i,n;
	Land *p;
	assert(edb!=NULL);
	r = newCollection(20,20);
	c = edb->land_data;
	n = getCollectionCount(c);
	for (i=0;i<n;i++) {
		p = (Land *) getCollectionItem(c, i);
		if (!p) continue;
		if (p->lnd_x!=x || p->lnd_y!=y) continue;
		addCollectionItem(r, p);
	}
	return r;
}

/* edb_getNukesInSector -- retrieve info about nukes in a sector */
Collection  *edb_getNukesInSector(edb, x, y)
	Empire_Database *edb;
	int x;
	int y;
{
	/* this will be faster when we implement a secondary index */
	Collection *c, *r;
	int i,n;
	Nuke *p;
	assert(edb!=NULL);
	r = newCollection(20,20);
	c = edb->nuke_data;
	n = getCollectionCount(c);
	for (i=0;i<n;i++) {
		p = (Nuke *) getCollectionItem(c, i);
		if (!p) continue;
		if (p->nuk_x!=x || p->nuk_y!=y) continue;
		addCollectionItem(r, p);
	}
	return r;
}


void edb_setPreimageBuffer(edb, buf, buflen)
	Empire_Database *edb;
	void *buf;
	int buflen;
{
	assert(edb!=NULL);
	edb->objbuffer = buf;		/* pre-update buffer */
	edb->objbuflen = buflen;	/* length of pre-update buffer */
}

struct genobject *edb_getPreimageBuffer(edb)
	Empire_Database *edb;
{
	return (struct genobject *) edb->objbuffer;
}



