#ifndef __EDBPROC_H__
#define __EDBPROC_H__

#define NoSysFuncs

#ifndef TOOLKIT
#define TOOLKIT
#endif
#include "prototype.h"
#include "misc.h"
#include "collect.h"
#include "index.h"

#ifndef SERVERTYPES
#include "solo/userflds.h"
#include "solo/altnames.h"
#include "solo/typedefs.h"
#else
#include "ship.h"
#include "land.h"
#include "plane.h"
#include "sect.h"
#include "nuke.h"
#include "item.h"
#include "product.h"
#include "nat.h"
#include "map.h"
#include "other.h"
#endif

/* these typedefs are for convenience */

typedef struct plnstr Plane;
typedef struct plchrstr Plane_Build;
typedef struct shpstr Ship;
typedef struct mchrstr Ship_Build;
typedef struct lndstr Land;
typedef struct lchrstr Land_Build;
typedef struct nukstr Nuke;
typedef struct nchrstr Nuke_Build;
typedef struct sctstr Sector;
typedef struct dchrstr Sector_Build;
typedef struct natstr Country;
typedef struct cmapstr Cmap;
typedef struct bmapstr Bmap;
typedef struct gameinfo Version;


typedef struct pchrstr Product_Class;
typedef struct ichrstr Item_Class;


#define EDB_OBJECT_BUFSZ	2048	/* initial size of preimage buffer */

typedef struct EDB_Str {
	/* all of these should probably be private to the database manager */
	/* do not reference them directly from your program 		   */
	/* Use the various access routines instead... */
	
	Collection *plane_build, *land_build, *ship_build,
		    *sector_build, *nuke_build,
		   *item_class, *product_class;
	Collection *plane_data,  *land_data,  *ship_data,
		   *sector_data, *nuke_data;
	Index      
		   *plane_data_index,  *land_data_index, 
		   *ship_data_index,   *sector_data_index,
		   *nuke_data_index,

		   *plane_coord_index,  *land_coord_index, 
		   *ship_coord_index,   *sector_coord_index,
		   *nuke_coord_index,
		   
	           *plane_build_index, *land_build_index, 
		   *ship_build_index,  *sector_build_index,
		   *nuke_build_index, 
		   *item_class_index,  *product_class_index;
	Collection *news_data, *trade_data,
		   *loan_data, *treaty_data;
	Index	   *news_data_index, *trade_data_index,
		   *loan_data_index, *treaty_data_index;
	Collection *commod_data;
	Index	   *commod_data_index;	
	Collection *game_data;
	Index	   *game_data_index;
	Collection *bmap_data;
	Index	   *bmap_data_index;
	Collection *map_data;
	Index	   *map_data_index;
	Collection *country_data;
	Index      *country_data_index;
	void *dictionary;	/* opaque to client */
	void *objbuffer;	/* pre-update buffer */
	int  objbuflen;		/* length of pre-update buffer */
} Empire_Database;


typedef struct {
	/* all of these should probably be private to the database manager */
	/* do not reference them directly from your program */
	struct genobject **results;
	int nresults;
	int cursor;
} Empire_Query;

typedef enum {	V_Plane_Build, V_Land_Build, 
		V_Ship_Build, V_Sector_Build,
		V_Nuke_Build,
		V_Plane_Data,  V_Land_Data,
		V_Ship_Data,  V_Sector_Data,
		V_Nuke_Data,
		V_Item_Class, V_Product_Class,
		V_Cmap_Data, V_Bmap_Data,
		V_Country_Data, V_Version_Data,
		V_News_Data, V_Trade_Data,
		V_Loan_Data, V_Treaty_Data,
		V_Commodity_Data
	     } EmpireObjectType;

typedef int (*Edb_ReadFunc) _PROTO((char *, int, int *));   /* buf, len, bytesread */
typedef int (*Edb_WriteFunc) _PROTO((char *, int, int *));  /* buf, len, byteswritten */
typedef void (*Edb_CallBack) _PROTO(( void *, void * ));    /* context, object */

#ifdef __cplusplus
extern "C" {

#endif


/* edb_new -- allocate a new database control structure */
Empire_Database *edb_new _PROTO(());

/* edb_delete -- destroy a database control structure and substructures */
void edb_delete _PROTO((Empire_Database *));

/* High level functions to save/load the entire database... */
/* You supply the read/write function                       */


/* edb_save -- save the database */
int edb_save _PROTO((Empire_Database  *edb, Edb_WriteFunc func));

/* edb_load -- load the database */
int edb_load _PROTO((Empire_Database *edb, Edb_ReadFunc func));


/*  High level database query functions  */


/* edb_new_query -- perform a query operation on empire database */
Empire_Query *edb_new_query _PROTO((Empire_Database *edb, char *query));

/* edb_first_item -- get first item in query result set */
void *edb_first_item _PROTO((Empire_Query *eq));

/* edb_next_item -- get next item in query result set */
void *edb_next_item _PROTO((Empire_Query *eq));

/* edb_free_query -- dispose of query result set */
void edb_free_query _PROTO((Empire_Query *eq));



/* Low level database query functions */


/* edb_getPlaneBuild -- retrieve info about a particular plane type */
Plane_Build *edb_getPlaneBuild _PROTO((Empire_Database *edb, int planeType));

/* edb_getLandBuild -- retrieve info about a particular land type */
Land_Build *edb_getLandBuild _PROTO((Empire_Database *edb, int landType));

/* edb_getShipBuild -- retrieve info about a particular ship type */
Ship_Build *edb_getShipBuild _PROTO((Empire_Database *edb, int shipType));

/* edb_getNukeBuild -- retrieve info about a particular nuke type */
Nuke_Build *edb_getNukeBuild _PROTO((Empire_Database *edb, int nukeType));

/* edb_getSectorBuild -- retrieve info about a particular sector type */
Sector_Build *edb_getSectorBuild _PROTO((Empire_Database *edb, int sectType));

/* edb_getItemClass -- retrieve info about a particular commod */
Item_Class *edb_getItemClass _PROTO((Empire_Database *edb, int itemType));

/* edb_getProductClass -- retrieve info about a particular product */
Product_Class *edb_getProductClass _PROTO((Empire_Database *edb, int productType));

/* edb_getPlane -- retrieve info about a particular plane */
Plane  *edb_getPlane _PROTO((Empire_Database *edb, int planeNo));

/* edb_getLand -- retrieve info about a particular land unit */
Land   *edb_getLand _PROTO((Empire_Database *edb, int unitNo));

/* edb_getShip -- retrieve info about a particular ship */
Ship   *edb_getShip _PROTO((Empire_Database *edb, int shipNo));

/* edb_getNuke -- retrieve info about a particular nuke */
Nuke   *edb_getNuke _PROTO((Empire_Database *edb, int nukeNo));

/* edb_getPlanesInSector -- retrieve info about a particular plane */
Collection  *edb_getPlanesInSector _PROTO((Empire_Database *edb, int x, int y));

/* edb_getLand -- retrieve info about a particular land unit */
Collection  *edb_getLandsInSector _PROTO((Empire_Database *edb, int x, int y));

/* edb_getShipsInSector -- retrieve info about a particular ship */
Collection  *edb_getShipsInSector _PROTO((Empire_Database *edb, int x, int y));

/* edb_getNuke -- retrieve info about a particular nuke */
Collection  *edb_getNukesInSector _PROTO((Empire_Database *edb, int x, int y));

/* edb_getSector -- retrieve info about a particular sector */
Sector *edb_getSector _PROTO((Empire_Database *edb, int x, int y));

/* edb_getCountry -- retrieve country info */
Country *edb_getCountry _PROTO((Empire_Database *edb));

/* edb_getVersion -- retrieve game version info */
Version *edb_getVersion _PROTO((Empire_Database *edb));

/* edb_getMap -- retrieve map info */
Cmap *edb_getCmap _PROTO((Empire_Database *edb, int x, int y));

/* edb_getBmap -- retrieve map info */
Bmap *edb_getBmap _PROTO((Empire_Database *edb, int row));

/* edb_enumerate -- enumerate all of a particular type of thing */
void edb_enumerate _PROTO((Empire_Database *edb, EmpireObjectType itemType,
		   Edb_CallBack func, void *context));


/* edb_parse_c_sync -- process a C_SYNC message and update the database */
struct genobject *edb_parse_c_sync _PROTO((Empire_Database *edb, s_char *msg));

/* edb_print_c_sync -- construct C_SYNC message for an object */
void edb_print_c_sync _PROTO((Empire_Database *edb, s_char *buf, void *object));

void edb_dumpHeaders _PROTO((FILE *, Empire_Database *));

void edb_dumpStructs _PROTO((FILE *, Empire_Database *));

void edb_setPreimageBuffer _PROTO((Empire_Database *, void *, int));

struct genobject *edb_getPreimageBuffer _PROTO((Empire_Database *));

#ifdef __cplusplus
}
#endif

#endif
